#include "Camera.h"
#include "Entity.h"

namespace MyEngine
{
	Camera::Camera( )
		: m_fYaw(0.0F),
		m_fPitch(0.0F),
		m_pos(D3DXVECTOR3(0.0F, 0.0F, 0.0F)),
		m_upVec(D3DXVECTOR3(0.0F, 1.0F, 0.0F)),
		m_fFOV(D3DX_PI / 3.0F),
		m_fAspectRatio(1.0F),
		m_fNearDist(1.0F),
		m_fFarDist(1500.0F),
		m_pEntity(NULL)
	{
	}

	void Camera::AttachToEntity(Entity* pEnt)
	{
		assert(pEnt != NULL);
		m_pEntity = pEnt;
	}

	void Camera::SetOrientation( const D3DXVECTOR3& Orientation )
	{
		//m_o
	}

	D3DXVECTOR3 Camera::Orientation() const
	{
		// /////////////////////////////////////////////////////////////////
		// / Construct a rotation matrix from Euler angles.
		// /////////////////////////////////////////////////////////////////
		D3DXMATRIX rotation;
		D3DXMatrixRotationYawPitchRoll(&rotation, Yaw(), Pitch(), 0.0F);

		// /////////////////////////////////////////////////////////////////
		// / Apply the rotation matrix to initial orientation.
		// /////////////////////////////////////////////////////////////////
		D3DXVECTOR3 orientation(0.0F, 0.0F, 1.0F);
		D3DXVec3TransformCoord(&orientation, &orientation, &rotation);

		return orientation;
	}

	void Camera::SetYaw(float angle)
	{
		m_fYaw = angle;

		if(m_pEntity != NULL)
		{
			D3DXVECTOR3 ang(m_pEntity->Angle());
			ang.x = m_fYaw;
			m_pEntity->SetAngle(ang);
		}
	}

	void Camera::SetPitch(float angle)
	{
		float fMaxPitch = D3DX_PI/2.1F;

		m_fPitch = angle;

		if(m_fPitch > fMaxPitch)
		{
			m_fPitch = fMaxPitch;
		}
		else if(m_fPitch < -fMaxPitch)
		{
			m_fPitch = -fMaxPitch;
		}
	}

	void Camera::Rotate(float deltaYaw, float deltaPitch)
	{
		SetYaw(m_fYaw + deltaYaw);
		SetPitch(m_fPitch + deltaPitch);
	}

	D3DXMATRIX Camera::ViewMatrix()
	{
		D3DXVECTOR3 lookAt = Position() + Orientation();

		D3DXMATRIX view;
		D3DXMatrixLookAtLH(&view, &m_pos, &lookAt, &m_upVec);

		return view;
	}

	D3DXMATRIX Camera::ProjectionMatrix()
	{
		D3DXMATRIX projection;

		D3DXMatrixPerspectiveFovLH(
			&projection,
			FOV(),
			AspectRatio(),
			NearPlaneDist(),
			FarPlaneDist());

		return projection;
	}

	void Camera::Update(float dt)
	{
		if(m_pEntity != NULL)
		{
			SetPosition(m_pEntity->Position());
		}
	}
}
