#ifndef _CAMERA_H
#define _CAMERA_H

#include "stdinclude.h"
#include <d3dx9.h>

namespace MyEngine
{
	class Entity;

	class Camera
	{
	public:
		Camera();

		void AttachToEntity(Entity* pEnt);

		void SetPosition(const D3DXVECTOR3& pos) { m_pos = pos; }
		D3DXVECTOR3 Position() const { return m_pos; }

		void SetOrientation(const D3DXVECTOR3& Orientation);
		D3DXVECTOR3 Orientation() const;

		void SetYaw(float angle);
		float Yaw() const { return m_fYaw; }

		void SetPitch(float angle);
		float Pitch() const { return m_fPitch; }

		void Rotate(float deltaYaw, float deltaPitch);

		void SetFOV(float angle) { m_fFOV = angle; }
		float FOV() const { return m_fFOV; }

		void SetAspectRatio(float aspectRatio) { m_fAspectRatio = aspectRatio; }
		float AspectRatio() const { return m_fAspectRatio; }

		void SetNearPlaneDist(float dist) { m_fNearDist = dist; }
		float NearPlaneDist() const { return m_fNearDist; }

		void SetFarPlaneDist(float dist) { m_fFarDist = dist; }
		float FarPlaneDist() const { return m_fFarDist; }

		D3DXMATRIX ViewMatrix();
		D3DXMATRIX ProjectionMatrix();

		void Update(float dt);

	private:
		float m_fYaw, m_fPitch;
		D3DXVECTOR3 m_pos, m_upVec;

		float m_fFOV, m_fAspectRatio, m_fNearDist, m_fFarDist;

		Entity* m_pEntity;
	};
}

#endif // _CAMERA_H
