#include "Entity.h"
#include "Model.h"

namespace MyEngine
{
	Entity::Entity()
	{
		m_pos = m_vel = m_accel = D3DXVECTOR3(0.0F, 0.0F, 0.0F);
		m_ang = m_angVel = m_angAccel = D3DXVECTOR3(0.0F, 0.0F, 0.0F);

		m_orientAxes[0] = D3DXVECTOR3(1.0F, 0.0F, 0.0F);
		m_orientAxes[1] = D3DXVECTOR3(0.0F, 1.0F, 0.0F);
		m_orientAxes[2] = D3DXVECTOR3(0.0F, 0.0F, 1.0F);

		m_bRenderable = true;
	}

	void Entity::SetAngle(const D3DXVECTOR3& ang)
	{
		m_ang = ang;

		D3DXVECTOR3 axes[3] = {
			D3DXVECTOR3( 1.0F, 0.0F, 0.0F ),
			D3DXVECTOR3( 0.0F, 1.0F, 0.0F ),
			D3DXVECTOR3( 0.0F, 0.0F, 1.0F )
		};

		D3DXMATRIX rotation;
		D3DXMatrixRotationYawPitchRoll(&rotation, m_ang.x, m_ang.y, m_ang.z);

		for(int i=0; i < 3; ++i)
			D3DXVec3TransformCoord(&m_orientAxes[i], &axes[i], &rotation);
	}

	void Entity::OrientationAxes(D3DXVECTOR3& xAxis, D3DXVECTOR3& yAxis, D3DXVECTOR3& zAxis)
	{
		D3DXVec3Normalize(&m_orientAxes[0], &m_orientAxes[0]);
		D3DXVec3Normalize(&m_orientAxes[1], &m_orientAxes[1]);
		D3DXVec3Normalize(&m_orientAxes[2], &m_orientAxes[2]);

		xAxis = m_orientAxes[0];
		yAxis = m_orientAxes[1];
		zAxis = m_orientAxes[2];
	}

	void Entity::Update(float dt)
	{
		SetVelocity(Velocity() + dt*Acceleration());
		SetPosition(Position() + dt*Velocity() - 0.5F*dt*dt*Acceleration());

		SetAngVelocity(AngVelocity() + dt*AngAcceleration());
		SetAngle(Angle() + dt*AngVelocity() - 0.5F*dt*dt*AngAcceleration());
	}

	void Entity::Render()
	{
		if(m_bRenderable)
		{
			LPDIRECT3DDEVICE9 device = m_model->GetDevice();

			// /////////////////////////////////////////////////////////////////
			// / Apply the world transform.
			// /////////////////////////////////////////////////////////////////
			D3DXMATRIX translation, rotation, scaling, world;

			D3DXMatrixTranslation(&translation, m_pos.x, m_pos.y, m_pos.z);
			D3DXMatrixRotationYawPitchRoll(&rotation, m_ang.x, m_ang.y, m_ang.z);
			D3DXMatrixIdentity(&scaling);

			world = scaling * rotation * translation;
			device->SetTransform(D3DTS_WORLD, &world);

			// /////////////////////////////////////////////////////////////////
			// / Draw the entity.
			// /////////////////////////////////////////////////////////////////
			for(uint i=0; i < m_model->MaterialCount(); ++i)
			{
				device->SetMaterial(m_model->Material(i));
				device->SetTexture(0, m_model->Texture(i));
				m_model->Mesh()->DrawSubset(i);
			}
		}
	}
}
