#include "Model.h"
#include "Logger.h"

namespace MyEngine
{
	Model::Model(LPDIRECT3DDEVICE9 pDevice)
		: m_pDevice(pDevice)
	{
	}

	Model::~Model()
	{
		SAFE_RELEASE(m_pMesh);

		for(uint i=0; i < m_vSubsets.size(); ++i)
			SAFE_RELEASE(m_vSubsets[i].Texture);
	}

	bool Model::Load(const TString& strFilename)
	{
		HRESULT hr;

		DWORD nMaterialCount = 0;
		LPD3DXBUFFER pMaterialBuffer = NULL;

		hr = D3DXLoadMeshFromX(
			strFilename.c_str(),
			D3DXMESH_MANAGED,
			m_pDevice,
			NULL,
			&pMaterialBuffer,
			NULL,
			&nMaterialCount,
			&m_pMesh);

		if(FAILED(hr))
		{
			Logger::Instance()->Write(_T("FAILED: Couldn't load mesh \"") +
				strFilename + _T("\"."));
			return false;
		}

		// Allocate memory for materials.
		m_vSubsets.resize(nMaterialCount);

		// Extract material properties and texture names from the d3dxMtrlBuffer
		LPD3DXMATERIAL pd3dxMaterials = (D3DXMATERIAL*)pMaterialBuffer->GetBufferPointer();

		for(uint i=0; i < nMaterialCount; ++i)
		{
			// Material
			m_vSubsets[i].Material = pd3dxMaterials[i].MatD3D;
			m_vSubsets[i].Material.Ambient = m_vSubsets[i].Material.Diffuse;

			// Texture
			if(pd3dxMaterials[i].pTextureFilename != NULL)
			{
				TCHAR szTextureFilename[512];

#ifdef _UNICODE
				MultiByteToWideChar(CP_ACP, 0, pd3dxMaterials[i].pTextureFilename, -1,
					szTextureFilename, 512);
#else
				lstrcpyn(szTextureFilename, pd3dxMaterials[i].pTextureFilename, 512);
#endif

				// Create a full path to texture
				TString::size_type lastBackslashPos;
				lastBackslashPos = strFilename.find_last_of('\\');

				TString strTexturePath = strFilename.substr(0, lastBackslashPos+1) +
					szTextureFilename;

				hr = D3DXCreateTextureFromFileEx(
					m_pDevice,
					strTexturePath.c_str(),
					D3DX_DEFAULT,
					D3DX_DEFAULT,
					D3DX_DEFAULT,
					0,
					D3DFMT_UNKNOWN,
					D3DPOOL_MANAGED,
					D3DX_DEFAULT,
					D3DX_DEFAULT,
					0,
					NULL,
					NULL,
					&m_vSubsets[i].Texture);

				if(FAILED(hr))
				{
					Logger::Instance()->Write(_T("FAILED: Couldn't load texture \"") +
						strTexturePath + _T("\"."));
				}
			}
		}

		SAFE_RELEASE(pMaterialBuffer);

		return true;
	}
}
