#ifndef _MODEL_H
#define _MODEL_H

#include "stdinclude.h"
#include <d3dx9.h>

namespace MyEngine
{
	class Model
	{
	public:
		Model(LPDIRECT3DDEVICE9 pDevice);
		~Model();

		bool Load(const TString& strFilename);

		uint MaterialCount() const { return m_vSubsets.size(); }

		const D3DMATERIAL9* Material(uint index) const
		{
			assert(index < MaterialCount());
			return &m_vSubsets[index].Material;
		}

		LPDIRECT3DTEXTURE9 Texture(uint index) const
		{
			assert(index < MaterialCount());
			return m_vSubsets[index].Texture;
		}

		LPD3DXMESH Mesh() const { return m_pMesh; }

		LPDIRECT3DDEVICE9 GetDevice() const { return m_pDevice; }

	private:
		LPDIRECT3DDEVICE9 m_pDevice;
		LPD3DXMESH m_pMesh;

		struct MeshSubset
		{
			MeshSubset() : Texture(NULL) {}
			D3DMATERIAL9 Material;
			LPDIRECT3DTEXTURE9 Texture;
		};

		std::vector<MeshSubset> m_vSubsets;
	};
}

#endif // _MODEL_H
